@@ -45,6 +45,7 @@
 ;; browse-url-lynx-*	 Lynx	     0
 ;; browse-url-grail      Grail       0.3b1
 ;; browse-url-mmm        MMM         ?
+;; browse-url-kfm        kde
 ;; browse-url-generic    arbitrary
 
 ;; [A version of the Netscape browser is now free software
@@ -534,6 +535,8 @@
 ;;      Replaced use of temporary-file-directory with call to 
 ;;      `temp-directory', change requires considering differences in
 ;;      autoloading, use gnuclient instead of gnudoit.
+;;1999-12-17 Neal Becker <nbecker@fred.net>
+;;      * Add kfm support
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Code:
@@ -576,6 +579,7 @@ regexp should probably be \".\" to specify a default browser."
                           :value browse-url-lynx-emacs)
            (function-item :tag "Grail" :value  browse-url-grail)
            (function-item :tag "MMM" :value  browse-url-mmm)
+           (function-item :tag "kfm" :value browse-url-kfm)
            (function-item :tag "Specified by `Browse Url Generic Program'"
                           :value browse-url-generic)
            (function-item :tag "Default Windows browser"
@@ -1360,6 +1364,14 @@ don't offer a form of remote control."
 	 browse-url-generic-program
 	 (append browse-url-generic-args (list url))))
 
+;;;###autoload
+(defun browse-url-kfm (url &optional new-window)
+  "Ask the kfm WWW browser to load URL.
+Default to the URL around or before point."
+  (interactive (browse-url-interactive-arg "kfm URL: "))
+  (message "Sending URL to kfm...")
+    (start-process "kfmclient" nil "kfmclient"	 "openURL" url))
+
 (provide 'browse-url)
 
 ;;; browse-url.el ends here
