@@ -1,6 +1,6 @@
 ;;; pop3.el --- Post Office Protocol (RFC 1460) interface
 
-;; Copyright (C) 1996,1997,1998 Free Software Foundation, Inc.
+;; Copyright (C) 1996, 1997, 1998 Free Software Foundation, Inc.
 ;; Copyright (C) 1997 Franklin Lee
 
 ;; Author: Richard L. Pieri <ratinox@peorth.gweep.net>
@@ -10,7 +10,11 @@
 ;; Maintainer:      Andy Piper <andy@xemacs.org>
 
 ;; Keywords: mail, pop3
-;; Version: 2.04
+;; Version: 2.06
+
+;; Sync'ed up with: official pop3.el version 1.3s.  
+;; This version is a fork of the original pop3.el and the changes have
+;; not been merged back in to that version due to political difficulties.
 
 ;; This file is part of XEmacs.
 
@@ -73,7 +77,7 @@
 
 (require 'mail-utils)
 
-(defconst pop3-version "2.03")
+(defconst pop3-version "2.06-xemacs")
 
 (defvar pop3-maildrop (or (user-login-name) (getenv "LOGNAME") (getenv "USER") nil)
   "*POP3 maildrop.")
@@ -182,61 +186,63 @@ Nil means no, t means yes, not-nil-or-t means yet to be determined.")
     (setq message-count (length message-list))
     (message (format "Retrieving message list...%d unread" message-count))
     ;; now get messages
-    (while (<= n message-count)
-      (setq msgid (car (car message-list)))
-      (setq msglen (cdr (car message-list)))
-      (setq message-list (cdr message-list))
-      ;; only retrieve messages matching our regexp or in the uidl list
-      (if (or (not msgid)
-	      ;; don't download messages that are too large
-	      (and pop3-maximum-message-size 
-		   (> msglen pop3-maximum-message-size))
-	      (and (or (null pop3-leave-mail-on-server)
-		       ;; we top messages that are longer than 10k
-		       ;; since retrieving a large message only to
-		       ;; ignore it is wasteful.
-		       (> msglen 10000))
-		   pop3-retr-regexp
-		   (not (string-match pop3-retr-regexp
-				      (pop3-top process msgid)))))
-	  (message (format "Ignoring message %d of %d from %s..."
-			   n message-count pop3-mailhost))
-	(message (format "Retrieving message %d of %d from %s..."
-			 n message-count pop3-mailhost))
-	(if (pop3-retr process msgid crashbuf)
-	    (setq retrieved-messages (cons msgid retrieved-messages))
-	  (message (format "Retrieving message %d of %d from %s...ignored"
-			   n message-count pop3-mailhost))))
-      
-      ;; deleted a whole bunch of stuff here that updates the crashbox
-      ;; incrementally. This is way slow and mostly uneccessary, gnus
-      ;; and others will simply slurp the whole mail buffer anyway so
-      ;; why bother?
-      (setq n (+ 1 n)))
-    ;; frob the crash buffer at the end
-    (set-buffer crashbuf)
-    (append-to-file (point-min) (point-max) crashbox)
-    ;; mark messages as read
-    (when pop3-leave-mail-on-server
-      (pop3-save-uidls))
-    ;; now delete the messages we have retrieved
-    (unless (and pop3-leave-mail-on-server (null pop3-delete-retrieved-mail))
-      (mapcar
-       '(lambda (n)
-	  (message (format "Deleting message %d of %d from %s..."
-			   n message-count pop3-mailhost))
-	  (pop3-dele process n)) retrieved-messages))
-    
-    (pop3-quit process)
+    (unwind-protect
+	(while (<= n message-count)
+	  (setq msgid (car (car message-list)))
+	  (setq msglen (cdr (car message-list)))
+	  (setq message-list (cdr message-list))
+	  ;; only retrieve messages matching our regexp or in the uidl list
+	  (if (or (not msgid)
+		  ;; don't download messages that are too large
+		  (and pop3-maximum-message-size 
+		       (> msglen pop3-maximum-message-size))
+		  (and (or (null pop3-leave-mail-on-server)
+			   ;; we top messages that are longer than 10k
+			   ;; since retrieving a large message only to
+			   ;; ignore it is wasteful.
+			   (> msglen 10000))
+		       pop3-retr-regexp
+		       (not (string-match pop3-retr-regexp
+					  (pop3-top process msgid)))))
+	      (message (format "Ignoring message %d of %d from %s..."
+			       n message-count pop3-mailhost))
+	    (message (format "Retrieving message %d of %d from %s..."
+			     n message-count pop3-mailhost))
+	    (if (pop3-retr process msgid crashbuf)
+		(setq retrieved-messages (cons msgid retrieved-messages))
+	      (message (format "Retrieving message %d of %d from %s...ignored"
+			       n message-count pop3-mailhost))))
+	  ;; deleted a whole bunch of stuff here that updates the crashbox
+	  ;; incrementally. This is way slow and mostly uneccessary, gnus
+	  ;; and others will simply slurp the whole mail buffer anyway so
+	  ;; why bother?
+	  (setq n (+ 1 n)))
+      ;; frob the crash buffer at the end
+      (set-buffer crashbuf)
+      (append-to-file (point-min) (point-max) crashbox)
+      ;; mark messages as read
+      (when pop3-leave-mail-on-server
+	(pop3-save-uidls))
+      ;; now delete the messages we have retrieved
+      (unless (and pop3-leave-mail-on-server (null pop3-delete-retrieved-mail))
+	(mapcar
+	 '(lambda (n)
+	    (message (format "Deleting message %d of %d from %s..."
+			     n message-count pop3-mailhost))
+	    (pop3-dele process n)) retrieved-messages))
+      (pop3-quit process))
     (kill-buffer crashbuf)
-    ))
+    )
+  t)
 
 (defun pop3-open-server (mailhost port)
   "Open TCP connection to MAILHOST.
 Returns the process associated with the connection."
   (let ((process-buffer
 	 (get-buffer-create (format "trace of POP session to %s" mailhost)))
-	(process))
+	(process)
+	(coding-system-for-read 'binary)
+	(coding-system-for-write 'binary))
     (save-excursion
       (set-buffer process-buffer)
       (erase-buffer))
@@ -266,8 +272,7 @@ Returns the process associated with the connection."
 ;;      (insert command "\r\n"))
     (setq pop3-read-point (point))
     (goto-char (point-max))
-    (process-send-string process command)
-    (process-send-string process "\r\n")
+    (process-send-string process (concat command "\r\n"))
     )
 
 (defun pop3-read-response (process &optional return)
@@ -284,7 +289,7 @@ Return the response string if optional second argument is non-nil."
       (setq match-end (point))
       (goto-char pop3-read-point)
       (if (looking-at "-ERR")
-	  (error (buffer-substring (point) (- match-end 2)))
+	  (signal 'error (list (buffer-substring (point) (- match-end 2))))
 	(if (not (looking-at "+OK"))
 	    (progn (setq pop3-read-point match-end) nil)
 	  (setq pop3-read-point match-end)
@@ -373,7 +378,15 @@ Also escape 'From ' after '\n\n' with '>' (for mbox)."
 	      (setq From_ (concat (substring From_ 0 (match-beginning 0))
 				  (substring From_ (match-end 0)))))
 	    (goto-char (point-min))
-	    (insert From_))))))
+	    (insert From_)
+	    (re-search-forward "\n\n")
+	    (narrow-to-region (point) (point-max))
+	    (let ((size (- (point-max) (point-min))))
+	      (goto-char (point-min))
+	      (widen)
+	      (forward-line -1)
+	      (insert (format "Content-Length: %s\n" size)))
+	    )))))
 
 ;; UIDL support, mostly copied from epop3
 
